@@ -1,13 +1,10 @@
 import os
 import sys
-import fabric
 import pwd
 import shutil
 import subprocess
 
-from paramiko.ssh_exception import AuthenticationException
-from invoke.exceptions import UnexpectedExit
-
+from cf_remote import aramid
 from cf_remote import log
 from cf_remote.utils import whoami
 
@@ -20,7 +17,7 @@ class LocalConnection:
     def __init__(self):
         self.ssh_user = pwd.getpwuid(os.getuid()).pw_name
 
-    def run(self, command, hide=False, pty=False):
+    def run(self, command, hide=False):
         return subprocess.run(command, capture_output=True, shell=True, text=True)
 
     def put(self, src):
@@ -30,9 +27,37 @@ class LocalConnection:
             shutil.copy(src, dst)
 
 
+class Connection:
+    def __init__(self, host, user, connect_kwargs=None):
+        self.ssh_host = host
+        self.ssh_user = user
+        self._connect_kwargs = connect_kwargs
+
+    def run(self, command, hide=False):
+        extra_ssh_args = []
+        if "key_filename" in self._connect_kwargs:
+            extra_ssh_args.extend(["-i", self._connect_kwargs["key_filename"]])
+
+        ahost = aramid.Host(self.ssh_host, self.ssh_user, extra_ssh_args)
+        results = aramid.execute([ahost], command, echo=(not hide))
+        return results[ahost][0]
+
+    def put(self, src):
+        dst = os.path.basename(src)
+        ahost = aramid.Host(self.ssh_host, self.ssh_user)
+        results = aramid.put([ahost], src, dst)
+        return results[ahost][0].retcode
+
+    def __enter__(self, *args, **kwargs):
+        return self
+
+    def __exit__(self, *args, **kwargs):
+        pass
+
+
 def connect(host, users=None):
-    log.debug(f"Connecting to '{host}'")
-    log.debug(f"users= '{users}'")
+    log.debug("Connecting to '%s'" % host)
+    log.debug("users= '%s'" % users)
     if "@" in host:
         parts = host.split("@")
         assert len(parts) == 2
@@ -55,19 +80,19 @@ def connect(host, users=None):
             users = [whoami()] + users
     for user in users:
         try:
-            log.debug(f"Attempting ssh: {user}@{host}")
+            log.debug("Attempting ssh: %s@%s" % (user, host))
             connect_kwargs = {}
             key = os.getenv("CF_REMOTE_SSH_KEY")
             if key:
                 connect_kwargs["key_filename"] = os.path.expanduser(key)
-            c = fabric.Connection(host=host, user=user, connect_kwargs=connect_kwargs)
+            c = Connection(host=host, user=user, connect_kwargs=connect_kwargs)
             c.ssh_user = user
             c.ssh_host = host
             c.run("whoami", hide=True)
             return c
-        except AuthenticationException:
+        except aramid.ExecutionError:
             continue
-    sys.exit(f"Could not ssh into '{host}'")
+    sys.exit("Could not ssh into '%s'" % host)
 
 
 # Decorator to make a function automatically connect
@@ -94,52 +119,54 @@ def scp(file, remote, connection=None, rename=None):
         with connect(remote) as connection:
             scp(file, remote, connection, rename)
     else:
-        print(f"Copying: '{file}' to '{remote}'")
+        print("Copying: '%s' to '%s'" % (file, remote))
         connection.put(file)
         if rename:
             file = os.path.basename(file)
             if file == rename:
                 return 0
-            print(f"Renaming '{file}' -> '{rename}' on '{remote}'")
-            ssh_cmd(connection, f"mv {file} {rename}")
+            print("Renaming '%s' -> '%s' on '%s'" % (file, rename, remote))
+            ssh_cmd(connection, "mv %s %s" % (file, rename))
     return 0
 
 
 def ssh_cmd(connection, cmd, errors=False):
     assert connection
-    try:
-        log.debug(f"Running over SSH: '{cmd}'")
-        result = connection.run(cmd, hide=True)
+
+    log.debug("Running over SSH: '%s'" % cmd)
+    result = connection.run(cmd, hide=True)
+    if result.retcode == 0:
         output = result.stdout.replace("\r\n", "\n").strip("\n")
-        log.debug(f"'{cmd}' -> '{output}'")
+        log.debug("'%s' -> '%s'" % (cmd, output))
         return output
-    except UnexpectedExit as e:
-        msg = f"Non-sudo command unexpectedly exited: '{cmd}'"
+    else:
+        msg = "Non-sudo command unexpectedly exited: '%s' [%d]" % (cmd, result.retcode)
         if errors:
-            print(e)
+            print(result.stdout + result.stderr)
             log.error(msg)
         else:
-            log.debug(str(e))
+            log.debug(result.stdout + result.stderr)
             log.debug(msg)
         return None
 
 
 def ssh_sudo(connection, cmd, errors=False):
     assert connection
-    try:
-        log.debug(f"Running(sudo) over SSH: '{cmd}'")
-        escaped = cmd.replace('"', r"\"")
-        sudo_cmd = f'sudo bash -c "{escaped}"'
-        result = connection.run(sudo_cmd, hide=True, pty=True)
+
+    log.debug("Running(sudo) over SSH: '%s'" % cmd)
+    escaped = cmd.replace('"', r"\"")
+    sudo_cmd = 'sudo bash -c "%s"' % escaped
+    result = connection.run(sudo_cmd, hide=True)
+    if result.retcode == 0:
         output = result.stdout.strip("\n")
-        log.debug(f"'{cmd}' -> '{output}'")
+        log.debug("'%s' -> '%s'" % (cmd, output))
         return output
-    except UnexpectedExit as e:
-        msg = f"Sudo command unexpectedly exited: '{cmd}'"
+    else:
+        msg = "Sudo command unexpectedly exited: '%s' [%d]" % (cmd, result.retcode)
         if errors:
-            print(e)
+            print(result.stdout + result.stderr)
             log.error(msg)
         else:
-            log.debug(str(e))
+            log.debug(result.stdout + result.stderr)
             log.debug(msg)
         return None
